/***************************************
*
* Logging_rexlang block
* Writing data to csv file
* (C) 2018
*
* This LoggingSystem is generating its own name automatically based on "FileNamePrefix" + "Date (& eventually Time)" stamp.
* 

****************************************/

//assigning inputs to variables, these variables are READ-ONLY
long input(0) year;
long input(1) month;
long input(2) day;
long input(3) hour;
long input(4) minute;
long input(5) second;
string input(6) name; //The prefix of the filename
string input(7) AutoFileName; //for an internal comparison between actual used and current (new) filename
string input(10) DataStringInput; //datastring that needs to be logged
string input(11) HeaderStringInput; //Header string that needs to be used as header for each file
long input(15) TRIGGER; //input pulse for logging (through internal pulse generator currently set for 60 seconds)

//assigning variables to outputs, these variables are WRITE-ONLY
long output(15) writeresult;
string output(14) TimeStamp;
string output(13) Filename;
string output(12) LastStoredString;

//variables declaration
long filehandle;

//the init procedure is executed once when the REXLANG function block initializes
long init(void)
{
	return 0;
}

//the main procedure is executed once in each sampling period
long main(void)
{
		long i;
		string s[1024];
		string logline[350];
		string s_month[20], s_day[20], s_hour[20], s_minute[20], s_second[20];		

		i = 1;
		s = "";
		s_month = long2str(month);
		if (strlen(s_month) == 1) s_month = "0" + s_month;
		s_day = long2str(day);
		if (strlen(s_day) == 1) s_day = "0" + s_day;
		s_hour = long2str(hour);
		if (strlen(s_hour) == 1) s_hour = "0" + s_hour;
		s_minute = long2str(minute);
		if (strlen(s_minute) == 1) s_minute = "0" + s_minute;
		s_second = long2str(second);
		if (strlen(s_second) == 1) s_second = "0" + s_second;		

		TimeStamp = s_hour + ":" + s_minute + ":" + s_second; // only for displaying timeStamp in SubSystem (under mask)
		
//If Date- or Time- stamp will change, the filename will be different, so it will generate a new file.
//Below you can make a selection WHEN the file name should automatically change

//Selection #1 = Every YEAR
//		Filename = name + "_" + long2str(year) + ".csv";	
//Selection #2 = Every MONTH
//		Filename = name + "_" + long2str(year) + s_month + ".csv";	
//Selection #3 = Every DAY
//		Filename = name + "_" + long2str(year) + s_month + s_day + ".csv";		
//Selection #4 = Every HOUR
		Filename = name + "_" + long2str(year) + s_month + s_day + s_hour + ".csv";		
//Selection #5 = Every MINUTE
//		Filename = name + "_" + long2str(year) + s_month + s_day + s_hour + s_minute + ".csv";	
//With the above selection, every hour, a new filename will be generated automatically.

//first open the file and reach its end


		filehandle = OpenFile(AutoFileName);
//The file "based on the ÄutoFileName string that comes in on input (7) of the REXLANG block will open.
//But if it did not exist and a new file will open, I would like to get a header line, (or multiple header lines) to better understand the parameters stored to a file when open in EXCEL.
//So this header string factually should be:
//
//HeaderString = "YYYY" + "," + "MM" + "," + "DD" + "," + "HH" + ":" + "MM" + ":" + "SS" + "," + "DataStringInput" + ","
// 
// But where to put this? The File is every time open and closed, so it only should apply this header line when the file is NEWLY generated.

		if (filehandle > 0)
		{
			while (i > 0)
			{
				i = Read(filehandle, s, 50);
				Trace(100, i);
			}
		}
		if (TRIGGER)
		{
//
//			logline = long2str(year) + "-" + s_month + "-" + s_day + " " + s_hour + ":" + s_minute + ":" + s_second + " Temperature is: " + double2str(temperature) + ";";
			logline = long2str(year) + "," + s_month + "," + s_day + "," + s_hour + ":" + s_minute + ":" + s_second + "," + DataStringInput + ",";
			Trace(123, "Writing to text file: " + logline);
			writeresult = Write(filehandle, logline + "\n");
			LastStoredString = long2str(year) + "," + s_month + "," + s_day + "," + s_hour + ":" + s_minute + ":" + s_second + "," + DataStringInput + ",";
		}

		if (filehandle > 0) Close(filehandle);

	return 0;
}

//the exit procedure is executed once when the task is correctly terminated
// (system shutdown, downloading new control algorithm, etc.)
long exit(void)
{
	if (filehandle > 0) Close(filehandle);
	return 0;
}
